package de.kompf.javaxml.jaxb.xsdfirst;

import java.io.*;
import javax.xml.bind.*;
import de.kompf.javaxml.jaxb.xsdfirst.data.*;

/**
 * Sample to marshall and unmarshall a GPX file using JAXB.
 * It uses the 'xsd-first' approach.
 * @author Kompf
 *
 */
public class GpxJaxb {

  private void printSummary(GpxType gpxFile, PrintStream out) {
    out.println("*** Summary ***");
    out.printf("%d way points\n", gpxFile.getWpt().size());
    for (WptType pt : gpxFile.getWpt()) {
      out.printf("  '%s' %.5f %.5f (%.1f)\n", pt.getName(), pt.getLat(), pt.getLon(), pt.getEle());
    }
    out.printf("%d routes\n", gpxFile.getRte().size());
    for (RteType route : gpxFile.getRte()) {
      out.printf("  Route '%s' has %d points\n", route.getName(), route.getRtept().size());
    }
    out.printf("%d tracks\n", gpxFile.getTrk().size());
    for (TrkType track : gpxFile.getTrk()) {
      out.printf("  Track '%s' has %d segments with ", track.getName(), track.getTrkseg().size());
      for (TrksegType trkseg : track.getTrkseg()) {
        out.printf("%d ", trkseg.getTrkpt().size());
      }
      out.println("points");
    }
  }

  private void writeGpx(GpxType gpxFile, File file) throws JAXBException {
    JAXBContext jc = JAXBContext.newInstance(GpxType.class);
    Marshaller m = jc.createMarshaller();
    m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
    ObjectFactory obf = new ObjectFactory();
    m.marshal(obf.createGpx(gpxFile), file);
  }
  
  private GpxType readGpx(File file) throws JAXBException {
    return JAXB.unmarshal(file, GpxType.class);
  }
  
  private void createRouteFromWaypoints(GpxType gpxFile) {
    RteType route = new RteType();
    gpxFile.getRte().add(route);
    route.setName("Created from waypoints");
    for (WptType pt : gpxFile.getWpt()) {
      route.getRtept().add(pt);
    }
  }
  
  public static void main(String[] args) throws Exception {
    GpxJaxb simpleGpxJaxb = new GpxJaxb();
    // read file with way points
    File file2 = new File("Rue-Mz.gpx");
    GpxType gpxType = simpleGpxJaxb.readGpx(file2);
    simpleGpxJaxb.printSummary(gpxType, System.out);
    // create route from waypoints
    simpleGpxJaxb.createRouteFromWaypoints(gpxType);
    // write
    File file3 = new File("Rue-Mz-Rte.gpx");
    simpleGpxJaxb.writeGpx(gpxType, file3);
    
    System.out.println("GPX file created: " + file3.getAbsolutePath());
  }
}
